/*
* file CcPolicy.java
*
* Licensed Materials - Property of IBM
* Restricted Materials of IBM
* 
* com.ibm.rational.wvcm.stp.cc.CcPolicy
*
* (C) Copyright IBM Corporation 2012.  All Rights Reserved. 
* Note to U.S. Government Users Restricted Rights:  Use, duplication or  
* disclosure restricted by GSA ADP  Schedule Contract with IBM Corp. 
*/
package com.ibm.rational.wvcm.stp.cc;

import static com.ibm.rational.wvcm.stpex.StpExBase.PROPERTY_NAMESPACE;

import java.util.List;
import java.util.Map;

import javax.wvcm.Feedback;
import javax.wvcm.PropertyNameList.PropertyName;
import javax.wvcm.Resource;
import javax.wvcm.ResourceList;
import javax.wvcm.WvcmException;

/**
 * <p>A proxy for a ClearCase policy.
 * 
 * <p>A policy has an Access Control List (ACL) for each controlled VOB metatype (such as UCM project, 
 * branch type, element). The entries in a policy list a principal and its 
 * granted permissions. Principals in a policy are usually roles, with the name of the role defined 
 * by the ClearCase administrator. You can also put in specific users or groups as well, but we recommend 
 * putting them into the rolemaps implementing the policy.
 * 
 * <p>Each ACL-enabled VOB has a default policy named <em>DefaultPolicy</em>.  This default 
 * policy grants the VOB owner full access; nobody else has any access.
 */
public interface CcPolicy extends CcTypeBase {

    /**
     * Map of names of VOB object proxy types to lists of access control entries which make up the 
     * permissions template this policy will apply to a rolemap.
     */
    PropertyName<Map<String, List<CcAccessControlEntry>>> ACL_MAP =
        new PropertyName<Map<String, List<CcAccessControlEntry>>>(PROPERTY_NAMESPACE, "policy-acl-map");

    /**
     * Get the value of this resource's {@link #ACL_MAP} property.
     * 
     * @return Map of names of VOB object proxy types to lists of access control entries 
     * which make up the effective ACLs that this policy would generate when combined
     * with a rolemap and applied to VOB objects.
     * @throws WvcmException
     */
    Map<String, List<CcAccessControlEntry>> getAclMap() throws WvcmException;

    /**
     * List of rolemaps which implement this policy.
     */
    PropertyName<ResourceList<CcRolemap>> ROLEMAP_LIST =
            new PropertyName<ResourceList<CcRolemap>>(PROPERTY_NAMESPACE, "policy-rolemap-list");
    
    /**
     * Get the value of this resource's {@link #ROLEMAP_LIST} property.
     * 
     * @return List of rolemaps which implement this policy.
     * @throws WvcmException
     */
    ResourceList<CcRolemap> getRolemapList() throws WvcmException;
    
    /**
     * Set the value of this proxy's {@link #ACL_MAP} property.
     * This property may only be set for use with the 
     * {@link #doCreateCcPolicy(com.ibm.rational.wvcm.stp.cc.CcTypeBase.TypeCreateFlag[], Feedback)}
     * call.
     * It cannot be written using {@link Resource#doWriteProperties(Feedback)}.
     * @throws WvcmException
     *         if this proxy doesn't define a value for this property.
     */
    void setAclMap(Map<String, List<CcAccessControlEntry>> aclMap) throws WvcmException;

    /**
     * <p>
     * Create a new policy at the location specified by this proxy. The
     * location should be an object name selector specifying the policy's name
     * and the repository (VOB) in which to create it.
     * </p>
     * <p>
     * Set the optional {@link #ACL_MAP} property to specify the principals and access
     * allowed for the proxy types in this policy.
     * If this property is not set, the policy will not provide any permissions.
     * 
     * </p>
     * You may optionally set the following additional properties prior to
     * calling this method:
     * <bl>
     * <li> {@link #COMMENT} </li>
     * <li> {@link CcTypeBase#SCOPE} <em>(Not yet supported.)</em>
     * </bl>
     * @param flags Resource-specific creation flags. <em>(The ACQUIRE flag is not yet supported.)</em>
     */
    public CcPolicy doCreateCcPolicy(TypeCreateFlag[] flags, Feedback feedback) throws WvcmException;
    
    /**
     * <p>
     * Modify an existing policy at the location specified by this proxy. The
     * location should be an object name selector specifying the policy's name
     * and the repository (VOB) in which to create it.
     * </p>
     * <p>
     * Set the optional {@link #ACL_MAP} property to modify the principals and access
     * allowed for the proxy types in this policy. If not specified,
     * the existing map will not be changed.
     * 
     * </p>
     * You may optionally set the following additional properties prior to
     * calling this method:
     * <bl>
     * <li> {@link #COMMENT} Provides a comment for the policy modification, 
     * without overriding the creation comment.</li>
     * </bl>
     * @param flags Resource-specific creation flags. <em>(The ACQUIRE flag is not yet supported.)</em>
     */
    public CcPolicy doModifyCcPolicy(TypeCreateFlag[] flags, Feedback feedback) throws WvcmException;
}
